@@ -1,6 +1,8 @@
 import logging
 from IPython.core.display import display, HTML
 from expai.utils import generate_response
+import numpy as np
+
 
 class ExpaiModelExplainer:
     def __init__(self, model_id: str, project_id: str, api_key: str, headers: dict, server_name: str, session, project):
@@ -11,7 +13,7 @@ class ExpaiModelExplainer:
         self.api_key = api_key
 
         self.headers = headers
-        
+
         self.sess = session
         self.project = project
 
@@ -37,19 +39,20 @@ class ExpaiModelExplainer:
                 return None
 
         json = {
-                    "sample_id": sample_id,
-                    "shap": shap,
-                    "lime": lime,
-                    "indexes": indexes
-                }
+            "sample_id": sample_id,
+            "shap": shap,
+            "lime": lime,
+            "indexes": indexes
+        }
 
         response = self.sess.request("GET",
-                                    self.server_name + "/api/explain/{}".format(self.model_id), headers=self.headers, json=json)
+                                     self.server_name + "/api/explain/{}".format(self.model_id), headers=self.headers, json=json)
 
         return generate_response(response, 'shap_values')
 
-    def explain_model(self, sample_name: str = None, sample_id: str = None, indexes: list = None, shap: bool = True, lime: bool = False):
+    def explain_model(self, sample_name: str = None, sample_id: str = None, indexes: list = None, shap: bool = True, lime: bool = False, target_class: str = None):
         assert sample_name is not None or sample_id is not None, "You must provide a sample name or id for the explanation"
+
         if indexes is None:
             logging.warning("You will generate explanation for all entries in the file. This might consume many credits and time. Use it on your own risk.")
 
@@ -62,18 +65,19 @@ class ExpaiModelExplainer:
                 return None
 
         json = {
-                    "sample_id": sample_id,
-                    "shap": shap,
-                    "lime": lime,
-                    "indexes": indexes
-                }
+            "sample_id": sample_id,
+            "shap": shap,
+            "lime": lime,
+            "indexes": indexes,
+            "target_class": target_class
+        }
 
         response = self.sess.request("GET",
-                                    self.server_name + "/api/explain/{}/graphics".format(self.model_id), headers=self.headers, json=json)
+                                     self.server_name + "/api/explain/{}/graphics".format(self.model_id), headers=self.headers, json=json)
 
         return generate_response(response, 'visualizations')
 
-    def explain_sample(self, sample_name: str = None, sample_id: str = None, index: list = None, shap: bool = True, lime: bool = False):
+    def explain_sample(self, sample_name: str = None, sample_id: str = None, index: list = None, shap: bool = True, lime: bool = False, target_class: str = None):
         assert sample_name is not None or sample_id is not None, "You must provide a sample name or id for the explanation"
         assert index is not None, "You must specify the index of the sample you want to explain"
 
@@ -86,14 +90,15 @@ class ExpaiModelExplainer:
                 return None
 
         json = {
-                    "sample_id": sample_id,
-                    "shap": shap,
-                    "lime": lime,
-                    "index": [index]
-                }
+            "sample_id": sample_id,
+            "shap": shap,
+            "lime": lime,
+            "index": [index],
+            "target_class": target_class
+        }
 
         response = self.sess.request("GET",
-                                    self.server_name + "/api/explain/{}/sample/graphics".format(self.model_id), headers=self.headers, json=json)
+                                     self.server_name + "/api/explain/{}/sample/graphics".format(self.model_id), headers=self.headers, json=json)
 
         if response.ok:
             return response.json()['visualizations']['single_force_plot']
@@ -106,7 +111,6 @@ class ExpaiModelExplainer:
                     response.json()['message']))
             return None
 
-
     def plot(self, plot_html):
         display(HTML(plot_html))
 
@@ -123,6 +127,12 @@ class ExpaiModelExplainer:
         if original_sample_display is not None or modified_sample_display is not None:
             assert original_sample_display is not None and modified_sample_display is not None, "You must provide both display samples."
 
+        # Ensure there are no integer keys in the dictionaries
+        for dict_ in [x for x in [original_sample, modified_sample, original_sample_display, modified_sample_display] if x is not None]:
+            for key, value in dict_.items():
+                if np.issubdtype(type(value), np.integer):
+                    dict_[key] = float(value)
+
         json = {
             "original_sample": original_sample,
             "modified_sample": modified_sample,
@@ -131,7 +141,7 @@ class ExpaiModelExplainer:
         }
 
         response = self.sess.request("GET",
-                                    self.server_name + "/api/explain/{}/what_if".format(self.model_id), headers=self.headers, json=json)
+                                     self.server_name + "/api/explain/{}/what_if".format(self.model_id), headers=self.headers, json=json)
 
         if response.ok:
             return response.json()['visualizations'], response.json()['shap_values']
@@ -143,6 +153,3 @@ class ExpaiModelExplainer:
                 logging.error("There was an error generating what if explanation. Please, check the details below. \n {}".format(
                     response.json()['message']))
             return None
-
-
-
