@@ -2,6 +2,9 @@ import requests
 import logging
 from expai.expai_model import ExpaiModelExplainer
 from expai.utils import generate_response
+import io
+import pandas as pd
+import ast
 
 
 class ExpaiProject:
@@ -56,8 +59,8 @@ class ExpaiProject:
 
 
     def create_model(self, model_path: str = None, is_supervised: bool = True, model_name: str = None,
-                     model_type: str = None, model_library: str = None, model_objective: str = None,
-                     model_prediction_type: str = None):
+                     model_category: str = None, model_library: str = None, model_objective: str = None,
+                     model_prediction_type: str = None, output_classes: list = None):
         """
         Create a new model for a project.
         :param model_path: path where your model is stored locally
@@ -67,7 +70,7 @@ class ExpaiProject:
         """
         assert model_name is not None, "To create a model you must give it a model_name"
         assert model_path is not None, "To create a model you must specify the path to the stored model"
-        assert model_type is not None, "Please, indicate the model_type. Refer to documentation for further information"
+        assert model_category is not None, "Please, indicate the model_category. Refer to documentation for further information"
         assert model_library is not None, "Please, indicate the model_library. Refer to documentation for further information"
         assert model_objective is not None, "Please, indicate the model_objective. Refer to documentation for further information"
         assert model_prediction_type is not None, "Please, indicate the model_prediction_type. Refer to documentation for further information"
@@ -75,10 +78,11 @@ class ExpaiProject:
 
         data = {"model_name_des": model_name,
                 "is_supervised_flag": 1 if is_supervised else 0,
-                'model_type_des': model_type,
+                'model_category_des': model_category,
                 'model_library_des': model_library,
                 'model_objective_des': model_objective,
-                'model_prediction_type_des': model_prediction_type}
+                'model_prediction_type_des': model_prediction_type,
+                'output_classes_des': ','.join(str(e) for e in output_classes) if output_classes is not None else None}
 
         files = {'model_file': open(model_path, 'rb')}
         headers = {'api_key': self.api_key}
@@ -129,6 +133,35 @@ class ExpaiProject:
         else:
             logging.error("We could not find your model. Please, check your name or try using model_id as parameter")
 
+    def update_model(self, model_name: str = None, model_id: str = None, update_info: dict = None):
+        """
+        Update a sample withing a project
+
+        :param sample_name:
+        :param sample_id:
+        :param update_info: information to be updated in a dictionary
+        :return:
+        """
+        assert model_id is not None or model_name is not None, "You must indicate the model_name or model_id"
+        assert update_info is not None, "You must provide the update_info dictionary with the values to be updated"
+
+        if model_id is None:
+            model_id = self._get_model_id_from_name(model_name)
+
+            if model_id is None:
+                logging.error(
+                    "We could not find any model matching that name. Please, try again or use model_id as parameter")
+                return False
+
+        response = self.sess.request("PATCH",
+                                    self.server_name + "/api/projects/{}/model/{}".format(self.project_id, model_id),
+                                    headers=self.headers, json=update_info)
+
+        if response.ok:
+            return "Model with id {} successfully updated".format(model_id)
+        else:
+            return generate_response(response)
+
     def _get_model_id_from_name(self, model_name: str = None):
 
         model_list = self.model_list()
@@ -175,7 +208,7 @@ class ExpaiProject:
             return generate_response(response, 'samples')
 
     def create_sample(self, sample_path: str = None, sample_name: str = None, sample_separator: str = None, sample_target_col: str = None,
-                      sample_encoding: str = 'utf-8', is_display = None, reference_sample_id: str = None, reference_sample_name: str = None):
+                      sample_encoding: str = 'utf-8', is_display = False, reference_sample_id: str = None, reference_sample_name: str = None):
         """
         Create a new sample for the project
         :param sample_target_col: target column for the model
@@ -225,7 +258,7 @@ class ExpaiProject:
         :param model_id: Id for the model you want to remove (optional)
         :return: True if successful
         """
-        assert sample_id is not None or sample_name is not None, "You must indicate the model_name or model_id"
+        assert sample_id is not None or sample_name is not None, "You must indicate the sample_name or sample_id"
 
         if sample_id is None:
             sample_id = self._get_sample_id_from_name(sample_name)
@@ -244,6 +277,87 @@ class ExpaiProject:
         else:
             return generate_response(response)
 
+    def update_sample(self, sample_name: str = None, sample_id: str = None, update_info: dict = None):
+        """
+        Update a sample withing a project
+
+        :param sample_name:
+        :param sample_id:
+        :param update_info: information to be updated in a dictionary
+        :return:
+        """
+        assert sample_id is not None or sample_name is not None, "You must indicate the sample_name or sample_id"
+        assert update_info is not None, "You must provide the update_info dictionary with the values to be updated"
+
+        if sample_id is None:
+            sample_id = self._get_sample_id_from_name(sample_name)
+
+            if sample_id is None:
+                logging.error(
+                    "We could not find any sample matching that name. Please, try again or use sample_id as parameter")
+                return False
+
+        response = self.sess.request("PATCH",
+                                    self.server_name + "/api/projects/{}/sample/{}".format(self.project_id, sample_id),
+                                    headers=self.headers, json=update_info)
+
+        if response.ok:
+            return "Sample with id {} successfully updated".format(sample_id)
+        else:
+            return generate_response(response)
+
+    def append_sample(self, sample_name: str = None, sample_id: str = None, sample_path: str = None, sample_encoding: str = 'utf-8'):
+        assert sample_id is not None or sample_name is not None, "You must indicate the sample_name or sample_id"
+        assert sample_path is not None, "Please, provide the local path for the sample you want to update"
+
+        if sample_id is None:
+            sample_id = self._get_sample_id_from_name(sample_name)
+
+            if sample_id is None:
+                logging.error(
+                    "We could not find any sample matching that name. Please, try again or use sample_id as parameter")
+                return False
+
+        files = {'sample_file': open(sample_path, 'r', encoding=sample_encoding)}
+
+        headers = {'api_key': self.api_key}
+
+        response = self.sess.request("POST", self.server_name + "/api/projects/{}/sample/{}/append".format(self.project_id, sample_id),
+                                     headers=headers, files=files)
+
+        if response.ok:
+            return "Successfully appended to sample with id {}".format(sample_id)
+        else:
+            return generate_response(response)
+
+    def get_sample(self, sample_name: str = None, sample_id: str = None, drop_target: bool = True):
+        assert sample_id is not None or sample_name is not None, "You must indicate the sample_name or sample_id"
+
+        if sample_id is None:
+            sample_id = self._get_sample_id_from_name(sample_name)
+
+            if sample_id is None:
+                logging.error(
+                    "We could not find any sample matching that name. Please, try again or use sample_id as parameter")
+                return False
+
+        response = self.sess.request("GET", self.server_name + "/api/projects/{}/sample/{}/get".format(self.project_id, sample_id), headers=self.headers)
+
+        if response.ok:
+            encoding = response.headers['encoding']
+            separator = response.headers['separator']
+            df = pd.read_csv(io.StringIO(response.content.decode(encoding)), index_col=0, sep=separator)
+            if drop_target is True:
+                try:
+                    drop_col = ast.literal_eval(response.headers['drop_cols'])
+                    df = df.drop(columns=drop_col)
+                except:
+                    logging.error("There was en error dropping the target column. Please, try setting drop_target to False")
+            return df
+        else:
+            return generate_response(response)
+
+
     def _get_sample_id_from_name(self, sample_name: str = None):
 
         sample_list = self.sample_list()
